/***************************************************************************
 *   Copyright (C) 2004 by Rick L. Vinyard, Jr.                            *
 *   rvinyard@cs.nmsu.edu                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Lesser General Public License as        *
 *   published by the Free Software Foundation version 2.1.                *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU Lesser General Public      *
 *   License along with this library; if not, write to the                 *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA              *
 ***************************************************************************/
#ifndef PAPYRUSHANDLEBOX_H
#define PAPYRUSHANDLEBOX_H

#include <papyrus/boxed.h>

namespace Papyrus {

  /**
    @author Rick L Vinyard Jr <rvinyard@cs.nmsu.edu>
  */
  class Handlebox : public Boxed {
    public:
      typedef enum HANDLE {
        HANDLE_TOP_CENTER,
        HANDLE_TOP_RIGHT,
        HANDLE_CENTER_RIGHT,
        HANDLE_BOTTOM_RIGHT,
        HANDLE_BOTTOM_CENTER,
        HANDLE_BOTTOM_LEFT,
        HANDLE_CENTER_LEFT,
        HANDLE_TOP_LEFT
    } HANDLE;

      typedef PapyrusSmartPointer<Handlebox> pointer;

      Handlebox(Handles handles=CORNER_HANDLES);

      static Handlebox::pointer create(Handles handles=CORNER_HANDLES);

      virtual ~Handlebox();

      void set_handle_shape( Drawable::pointer );
      void set_handle_shape( unsigned short int which, Drawable::pointer );
      void set_corner_handle_shape( Drawable::pointer );
      Drawable::pointer get_handle_shape( unsigned short int which );

      void clear_handles();
      void clear_handle( unsigned int which );

      int inside_handle( double x, double y );

      PAPYRUS_CLASS_NAME("Handle Box");

      PAPYRUS_CLONE_METHOD( Handlebox );

    protected:
      Drawable::pointer m_handle_shape[ 8 ];

      virtual void draw( Cairo::RefPtr< Cairo :: Context > cairo );

      /**
       * Reimplements Boxed' behavior when extents need to be updated.
       * Performs same behavior, but also adjusts handle shapes to their proper places.
       */
      virtual void update_extents();

      virtual void position_handle( Drawable::pointer, double x, double y );

  };

}

#endif
